#!/bin/sh

# bb-local.sh
#
# BIG BROTHER MONITORING SCRIPT
# Sean MacGuire - The MacLawran Group Inc.
# Version 1.2a
# May 4th, 1999
#
# This program is Copyright (c) 1997-1999
# The MacLawran Group Inc.
# All Rights Reserved
#
# For each system we monitor we watch
#	disk	USES df 
#	cpu	USES uptime
#	procs	CHECKS A LIST
# 	msgs	LOOKS FOR WARNINGS
#
# For the network we care about
#	telnet	USES telnet and checks the results
#	http	USES lynx to check for valid output
#
# THEN SEND THE RESULTS TO THE DISPLAY SERVER USING bb

# echo "***** BBHOME IS SET TO $BBHOME"
# echo "***** BBTMP IS SET TO $BBTMP"

if test ! "$BBTMP"                      # GET DEFINITIONS IF NEEDED
then
	 # echo "*** LOADING BBDEF ***"
        . $BBHOME/etc/bbdef.sh          # INCLUDE STANDARD DEFINITIONS
fi

#
#=====================================================================
# DISK TEST
#=====================================================================
#
# echo "*** DISK TEST 1 ***"
COLOR="green"
LINE="
"

$DF | $GREP $DFUSE | $GREP -v $DFEXCLUDE | $SORT +${DFSORT} > $BBTMP/DFOUT
cat $BBTMP/DFOUT | {
while read line
do
    set $line
    DISKNAME=$6
    DISKDEV=$1
    DISKPCT=`echo $5 | $SED "s/%//"`

    # echo "*** DISK TEST 2 ***"
    #
    # SUPPORT FOR NON-STANDARD df'S
    # Richard Dansereau <dansereau.richard@regis.uqam.ca>
    # Thanks to Thomas Alex <thomas.alex@commgensys.com>
    #
    case "$DFOUTPUT"
    in
        SCO3)   DISKNAME=$1
                DISKPCT=`echo $6 | $SED "s/%//"`
		DISKDEV=$7	# Is this true ?
		;;
	IRIX)   # Thanks to:    David Brandon
		DISKNAME=$7
		DISKPCT=`echo $6 | $SED "s/%//"`
		;;
    esac

    # echo "$DISKNAME $DISKPCT"

	DFWARNVAL=$DFWARN               # USE DEFAULT WARN VALUE
	DFPANICVAL=$DFPANIC             # USE DEFAULT PANIC VALUE

	#
	# NEW IN 1.08a
	# Robert Andre Croteau's custom disk file
	# IF WE HAVE A CUSTOM PARTITION FILE, USE IT.
	# DEFAULTS TO etc/bb-dftab
	# FORMAT:	partition:warn%:panic%
	# I.E.:		/usr:96:99
	#
	if test -f "${BBDFTAB}"
	then
		# echo "*** CHECKING FOR PARTITION $DISKNAME IN $BBDFTAB"
		DISKLINE=`$GREP "^${DISKNAME}:" $BBDFTAB`
		# echo "*** DISKLINE: $DISKLINE"
		if test "$DISKLINE" != ""
		then
			# echo "*** DISKLINE NOT NULL"
			set `echo $DISKLINE | $SED "s/:/ /g"`
        		DFWARNVAL=$2            # GET CURRENT FS WARNING LEVEL
        		DFPANICVAL=$3           # GET CURRENT FS PANIC LEVEL
			# echo "*** WARN AT $DFWARNVAL - PANIC AT $DFPANICVAL"
		fi
	fi

    	MARK=""
    	if test "$DISKPCT" -ge "$DFWARNVAL"         # ABOVE WARNING LEVEL
    	then
        	if test "$DISKPCT" -ge "$DFPANICVAL"    # ABOVE PANIC, RED ALERT !!!
        	then
            		COLOR="red"
            		MARK="PANIC"
	    		MARKLEVEL=$DFPANICVAL
	    		HIGHMARK="PANIC"
			GIFCOLOR="&red"
        	else
            		if test "$COLOR" = "green"          # ONLY IF NOT ALREADY SET
            		then
                		COLOR="yellow"
	        		HIGHMARK="WARNING"
            		fi
            		MARK="WARNING"
	    		MARKLEVEL=$DFWARNVAL
			GIFCOLOR="&yellow"
        	fi
        	LINE="${LINE}
${GIFCOLOR} ${DISKNAME} (${DISKPCT}%) has reached the defined disk space ${MARK} level (${MARKLEVEL}%)"
    	fi
done

#
# NOW SEND THIS INFORMATION TO THE BIG BROTHER DISPLAY UNIT
#
if [ "$COLOR" = "green" ]
then
    LINE="status $MACHINE.disk $COLOR `date` - Disk partitions on $MACHINE OK

`cat $BBTMP/DFOUT`"
else
    LINE="status $MACHINE.disk $COLOR `date` - Disk on $MACHINE at $HIGHMARK level $LINE

`cat $BBTMP/DFOUT`"
fi

$BB $BBDISP "$LINE"

$RM -f $BBTMP/DFOUT

}
#
#=====================================================================
# CPU TEST AND UPTIME WARNINGS
# AUTOMATIC WARNING IF THE SYSTEM IS UP FOR LESS THAN 1 HOUR
#=====================================================================
#
# echo "*** CPU TEST ***"
set `$UPTIME`

echo $3 | $GREP "," > /dev/null 2>&1
if test "$?" = 0
then
	UP=`echo $3 | $SED "s/,//"`
else
	UP=`echo $3 $4 | $SED "s/,//"`
fi

shift;shift;shift;shift;

ARG=`$EXPR $# - 2`			# 5 MINUTE LOAD AVERAGE
COUNT=1
while test "$COUNT" -le "$ARG"
do
	shift;
	COUNT=`$EXPR $COUNT + 1`
done

AVG=`echo $1 | $SED "s/\,//g" | $SED "s/\.//" `
AVG=`$EXPR $AVG + 0`
# echo "LOAD AVERAGE: $AVG"

LINE="LOAD AVG on $MACHINE is ${AVG}"

if test "$AVG" -ge "$CPUPANIC"
then
	COLOR="red"
elif test "$AVG" -ge "$CPUWARN"
then
	COLOR="yellow"
else	
	COLOR="green"
fi

# echo "*** PS COMMAND $PS ***"

$PS > $BBTMP/bb.$$			# GET A PS LISTING
P=`$CAT $BBTMP/bb.$$ | $WC`		# NUMBER OF PROCESSES
P=`echo $P`
U=`$WHO | $WC`				# GET NUMBER OF USERS
U=`echo $U`

#
# AUTOMATIC WARNING IF A SYSTEM HAS BEEN UP LESS THAN AN HOUR
# THIS WILL HELP US NOTICE SPONTANEOUS REBOOTS!
#
echo $UP | $GREP "min" > /dev/null 2>&1
if test "$?" = "0"
then
	COLOR="yellow"
fi

#
# NOW SEND THIS INFORMATION TO THE BIG BROTHER DISPLAY UNIT
#
$BB $BBDISP "status $MACHINE.cpu $COLOR `date` up: $UP, $U users, $P procs, load=${AVG}"

#
#=====================================================================
# PROCESSES THAT MUST EXIST
#=====================================================================
#
# echo "*** PROCESSES TEST ***"

LINE=""
STATLINE="No processes to check"
COLOR="green"

for proc in $PROCS $PAGEPROC		# SMM CHECK BOTH
do
				# No need to exclude grep from
				# process list
				# thanks to Paul Gluhosky
				# <paul.gluhosky@yale.edu>
	$GREP "$proc" $BBTMP/bb.$$ > /dev/null 2>&1
	if test "$?" != "0"
	then
		STATLINE="Some processes are DOWN"
		#
		# NOW SEE IF THIS IS A PROCESS WE SHOULD NOTIFY ON...
		#
		echo "$PAGEPROC" | $GREP "$proc" > /dev/null 2>&1
		if test "$?" = "0"
		then
			COLOR="red"
			LINE="$LINE
&red $proc DOWN"
		else
			if test "$COLOR" != "red"	# WASN'T A PANIC
			then
				COLOR="yellow"		# SO JUST WARN
			fi
			LINE="$LINE
&yellow $proc DOWN"
		fi	
	else
		if [ "$COLOR" = "green" ]
		then
			STATLINE="All processes are UP"
		fi
		LINE="$LINE
&green $proc UP"
	fi
done

#
# NOW SEND THIS INFORMATION TO THE BIG BROTHER DISPLAY UNIT
#
$BB $BBDISP "status $MACHINE.procs $COLOR `date` $STATLINE $LINE"
$RM -f $BBTMP/bb.$$

#
#=====================================================================
# NOTIFY ON BAD MESSAGES
#=====================================================================
#
# echo "*** MESSAGES TEST ***"

COLOR="green"
MSGLINE="No bad messages"
BADLINE=""

msgsfile=$BBTMP/BBMSG.$$
for word in $MSGS
do
        $GREP "$word" $MSGFILE > $msgsfile
        if [ "$?" -eq 0 ]                       # FOUND A BAD WORD
        then
                OLDIFS=$IFS
                IFS=";"
                for ignmsg in $IGNMSGS
                do
                        # Save in a file to keep new lines
                        $GREP -v "$ignmsg" $msgsfile > ${msgsfile}.tmp
                        mv ${msgsfile}.tmp ${msgsfile}
                done
                IFS=$OLDIFS
                if [ ! -s "${msgsfile}" ]
                then
                        continue
                fi
                LINE=`cat ${msgsfile} | $TAIL -10`
		MSGLINE="$MACHINE - bad message(s) found:

"
		BADLINE="${BADLINE}${LINE}"
		echo "$PAGEMSG" | $GREP "$word" > /dev/null 2>&1
		if test "$?" = "0"
		then
			COLOR="red"
			break;			# ENOUGH ALREADY...
		else
			if test "$COLOR" = "green"
			then
				COLOR="yellow"
			fi
		fi
	fi
done

$RM -f ${msgsfile}

$BB $BBDISP "status $MACHINE.msgs $COLOR `date` ${MSGLINE}${BADLINE}"
