#!/bin/sh

# bb-network.sh
#
# BIG BROTHER NETWORK MONITORING SCRIPT
# Sean MacGuire - The MacLawran Group Inc.
# Version 1.2a
# May 4th, 1999
#
# This program is Copyright (c) 1997-1999
# The MacLawran Group Inc.
# All Rights Reserved
#
# SEND THE TEST RESULTS TO THE DISPLAY SERVER USING bb

# echo "***** BBHOME IS SET TO $BBHOME IN bb-net"
if test ! "$LYNX"			# GET DEFINITIONS IF NEEDED
then
	. $BBHOME/etc/bbdef.sh		# INCLUDE STANDARD DEFINITIONS
fi

#
# SMM - NEW FOR 1.05
# ALLOW TESTING OF BANKS OF CONNECTIONS ONLY
# FORMAT OF THE LINE IS
# dialup name IP-ADDRESS COUNT
# dialup modem-bank1 132.220.50.1 16
#
$GREP "^dialup" $BBHOSTS |
while read line
do
	set $line
	if test "$#" != "4"
	then
		echo "MISSING ARGUMENTS FOR dialup"
		echo "Line format is:"
		echo "dialup pool-name start-ip count"
		exit 1
	fi
	NAME="$2"
	TTL="$4"
	set `echo $3 | $SED "s/\./ /g"`
	BASE="$1.$2.$3"
	START="$4"
	END=`$EXPR $START + $TTL - 1`
	# echo "NAME IS $NAME"
	# echo "TTL IS $TTL"
	# echo "BASE IS $BASE"
	# echo "START IS $START"
	# echo "END IS $END"
	LINE="FROM $BASE.$START TO $END DATA "
	while test "$START" -le "$END"
	do
		TEST="${BASE}.$START"
		# echo $TEST
		$PING $PINGPAR1 $TEST $PINGPAR2 > $BBTMP/BBPING 2>&1
		$GREP "bytes from" $BBTMP/BBPING > /dev/null 2>&1
		if test $? != "0"
		then
			LINE="$LINE clear "
		else
			LINE="$LINE green "
		fi
		START=`$EXPR $START + 1`
	done
	# echo "LINE IS $LINE"
	$BB $BBDISP "status dialup.${NAME} green `date` $LINE" 
done

$GREP "^[0-9]" $BBHOSTS |
while read line
do
	set $line
	HOSTIP=$1			# HOST IP ADDRESS
	IPADDR=`echo $1 | $SED "s/\.//g"`
	#
	# SMM - NEW FOR 1.07c
	# IMPLICIT SUPPORT FOR dhcp STATIONS WHEN LISTED AS 0.0.0.0
	# BUT THE PING TEST MUST BE AUTOMATICALLY DISABLED (noping)
	#
	if test "$HOSTIP" = "0.0.0.0"
	then
		line="$line noping"	# DISABLE THE PING TEST
	fi
	FULLNAME=$2
	#
	# SMM - NEW FOR 1.04
	# USE FULLY QUALIFIED MACHINE NAMES?
	# DOT -> COMMA -> DOT FOR WEB DISPLAY
	#
	if test "$FQDN" = "TRUE"
	then
		set `echo $FULLNAME | $SED "s/\./,/g"`
	else
		set `echo $FULLNAME | $SED "s/\./ /"`
	fi
	MACHINE=$1

	#
	# SMM 1.09d - SUPPORT FOR ALIAS DISPLAY
	#
	echo "$line" | $GREP "[ 	]testip" > /dev/null 2>&1
	if [ "$?" = "0" ]
	then
		HOSTSTRING=$HOSTIP
	else
		HOSTSTRING=$FULLNAME
	fi

	#
	#================================================
	# CONNECTIVITY TEST
	# Added support for non-standard pings. gunnar@bitcon.no
	#
	# SMM: PING BY IP ADDRESS INSTEAD OF BY NAME. (v1.04a)
	# SMM: TRY THREE TIMES BEFORE PAGING (v1.06b)
	# 	BB SHOULD NOT BE A 3AM WAKE UP CALL...
	#================================================

	#
	# SMM 1.07
	# TO PING OR NOT TO PING
	# FIXED FALSE POSITIVES THANKS TO consult@netcom.com
	#
	echo "$line" | $GREP "noping" > /dev/null 2>&1
	ERR=$?
	if test $ERR = "0"
	then
		COLOR="clear"			# PING TEST DISABLED BY REQUEST
		LINE="Ping test disabled by request (noping)"
	else
		COLOR="green"			# INNOCENT UNTIL PROVEN GUILTY...
		LINE="Connection OK"
		$PING $PINGPAR1 $HOSTIP $PINGPAR2 > $BBTMP/BBPING 2>&1
		ERR=$?
		if test $ERR = "0"		# DID IT REALLY PASS THE FIRST TEST...
		then
			$GREP "bytes from" $BBTMP/BBPING > /dev/null 2>&1
			ERR="$?"
		fi
	fi
	if test $ERR != "0"
	then
		echo "$line" | $GREP "dialup" > /dev/null 2>&1
		if test $? = "0"
		then
			COLOR="clear"
			LINE="bb-network.sh: $FULLNAME offline (dialup)"
		else		# NON-DIALUP GETS 3rd STRIKE...
			sleep 1		# PAUSE... AND TRY AGAIN...
			$PING $PINGPAR1 $HOSTIP $PINGPAR2 > $BBTMP/BBPING 2>&1
			ERR=$?
			if test $ERR = "0"		# DID IT REALLY PASS THE SECOND TEST...
			then
				$GREP "bytes from" $BBTMP/BBPING > /dev/null 2>&1
				ERR=$?
			fi
			if test $ERR != "0"
			then
				sleep 2
				LINE="bb-network.sh: Can't ping $FULLNAME..."
				$PING $PINGPAR1 $HOSTIP $PINGPAR2 > $BBTMP/BBPING 2>&1
				ERR=$?
				if test $ERR = "0"	# PASSED THE THIRD TEST...
				then
					$GREP "bytes from" $BBTMP/BBPING >/dev/null 2>&1
					ERR=$?
				fi
                                if test $ERR != "0"     # THREE STRIKES...
                                then 
                                        echo "$NONETPAGE" | $GREP "ping" >/dev/null 2>&1
                                        if test $? = "1"        # NOT FOUND - PAGE
                                        then
						COLOR="red"
                                        else
                                        	COLOR="yellow"
                                        	LINE="$LINE - configured not to page"
					fi
                                else    # OK ON THIRD TRY... CAME BACK...
                                        COLOR="yellow"
                                        LINE="Connection OK on 3rd try..."
                                fi
			else	# OK ON THIRD TRY... CAME BACK...
				COLOR="yellow"	
				LINE="Connection OK on 2nd try..."
			fi
		fi
	fi
	$BB $BBDISP "status ${MACHINE}.conn $COLOR `date` $LINE" 
	$RM -f $BBTMP/BBPING		# GET RID OF THE LEFTOVERS

	#
	# THIS IS NEW WITH 1.04... TEST FOR OTHER SERVICES HERE...
	# SEND THE RESULTS TO THE USUAL PLACE...
	#
	set $line			# GET ALL THE LINE ARGS
	shift; shift;			# SKIP THE IP-ADDR AND NAME

	while test "$1"			# AS LONG AS WE HAVE ARGS
	do
		line="$*"		# SAVE THIS..

		case $1 
		in

		#
		# Add telnet test Tony DeJoie (tony@bellcore.com)
		# Add ssh test Kim Shrier (kim@tinker.com)
		#
		ssh* | telnet* | nntp* | ftp* | pop* | smtp* | imap* )	# OUR SERVICES
			#
			# OK... GET THE SERVICE NUMBER OUT OF /etc/services
			# CAN BE SEPERATED BY SPACES/TABS... (v1.04a)
			#
			OLDIFS=$IFS
			IFS=":"
			set $1
			IFS=$OLDIFS
			SVCNAME=$1
			USEIP=$2
			SVC=""
			#
			# SMM 1.07a - 21 JUN 98
			# A LITTLE FRIENDLY CHECK HERE WILL SAVE LOTS OF GRIEF
			#
			SVCLINE=`$GREP "^${SVCNAME}[ 	]" /etc/services | $GREP "/tcp"`
			if test "$SVCLINE" = ""	# HOUSTON, WE HAVE A PROBLEM
			then
				echo "bb-network.sh: ERROR on bb-hosts line: $FULLNAME"
				echo "$SVCNAME NOT in /etc/services (CHECK SPELLING)"
				set $line
				shift;
				continue;
			fi

			set $SVCLINE

			while test "$SVC" = ""
			do
				echo $1 | $GREP "/tcp" > /dev/null
				if test "$?" = "0"
				then
					SVC=`echo $1 | $SED "s/\/tcp//"`
					#echo "FOUND SERVICE: $SVCNAME: $SVC"
				fi
				shift;
			done
			# echo "SETTING UP PROTOCOL SPECIFIC DATA"
			# By default bbnet sends "quit"
			case $SVCNAME
			in
				imap* )
					textmsg="* LOGOUT"
					;;
				*)
					textmsg="DONTUSEARGS"
					;;
			esac
			# echo "TESTING SERVICE $SVC"
			# if "" don't send any argument, use bbnet default
			RC=0
			if [ "$textmsg" != "DONTUSEARGS" ]
			then
				OUTPUT=`$BBHOME/bin/bbnet ${HOSTSTRING}:${SVC} "$textmsg"` 
				RC=$?
			else
				OUTPUT=`$BBHOME/bin/bbnet ${HOSTSTRING}:${SVC}` 
				RC=$?
			fi
			#
			# SMM - 1.09d
			# SUPPORT SERVICES EVEN WHEN THEY'RE NOT IN THE DNS
			#
			if test "$RC" = "1" -a "$HOSTSTRING" = "$FULLNAME"	# HOST NOT FOUND... TRY IP...
			then
				if [ "$textmsg" != "DONTUSEARGS" ]
				then
					OUTPUT=`$BBHOME/bin/bbnet ${HOSTIP}:${SVC} "$textmsg"` 
					RC=$?
				else
					OUTPUT=`$BBHOME/bin/bbnet ${HOSTIP}:${SVC}` 
					RC=$?
				fi
				OUTPUT="$OUTPUT
*** Note: $FULLNAME not found in DNS... Tested using IP addr directly..."
			fi

			if test "$RC" != "0"
			then
				# echo "SERVICE IS BROKEN"
				#
				# IF IT'S BROKEN, THEN SCREAM IF WE HAVE TO
				#
                		LINE="SERVER - $SVCNAME DOWN"

				#
				# NO MORE RED DOTS FOR DIALUP NETWORK STUFF
				#
				echo "$line" | $GREP "dialup" > /dev/null 2>&1
				if test $? = "0"
				then
                                	COLOR="clear"
				else

					#
					# SMM NEW IN 1.07b
					# CORRECT TEST FOR NETWORK SERVICE PAGING
					# THANKS TO: Patrick Hopewell <p.hopewell@utoronto.ca>
					#
					echo "$NONETPAGE" | $GREP "$SVCNAME" > /dev/null 2>&1
					if test $? = "1"	# NOT FOUND - PAGE
                			then
                        			COLOR="red"
					else
                        			COLOR="yellow"
						LINE="$LINE - configured not to page"
					fi
				fi
			else
				LINE="$SVCNAME OK: $OUTPUT"
				COLOR="green"
				# echo $OUTPUT
			fi
			$BB $BBDISP "status ${MACHINE}.${SVCNAME} $COLOR `date` $LINE" 
			;;

		#
		# NEW IN 1.06C 
		# DNS TESTING FROM Frank T. Kujawski <frank@bluegrass.net>
		#
		dns )	# echo "looking for domain server"
			if test "$HOSTIP" = "0.0.0.0"	# SMM 1.10 DDNS
			then
				$NSLOOKUP ${FULLNAME} ${FULLNAME} > /dev/null
			else
				$NSLOOKUP ${FULLNAME} ${HOSTIP} > /dev/null
			fi
			if test "$?" != "0"
			then
				LINE="DNS Server on ${FULLNAME} down"
				# IT HELPS TO SEARCH FOR THE RIGHT SERVICE!
				# Takahiro Yugawa <yugawa@itjit.ad.jp>
				echo "$NONETPAGE" | $GREP "dns" > /dev/null 2>&1
				if test $? = "1"	# NOT FOUND - PAGE
				then
					COLOR="red"
				else
					COLOR="yellow"
                                	LINE="$LINE - configured not to page"
				fi
			else
				COLOR="green"
				LINE="DNS Server OK"
			fi

			$BB $BBDISP "status ${MACHINE}.dns $COLOR `date` $LINE"

                        ;;

		http* )	# echo "DOING HTTP TEST..."
			HTTP=$1
			$LYNX $HTTP > $BBTMP/HTTP 2>&1
			if test "$?" != 0
			then
				LINE="bb-network.sh: Lost HTTP connection to $FULLNAME"
				echo "$NONETPAGE" | $GREP "http" > /dev/null 2>&1
				if test $? = "1"	# NOT FOUND - PAGE
				then
					echo "$line" | $GREP "dialup" > /dev/null 2>&1
					if test $? = "0"
					then
						COLOR="yellow"
                                		LINE="${LINE} (no page on dialup)"
					else
						COLOR="red"
					fi
				else
					COLOR="yellow"
                                	LINE="${LINE} - configured not to page"
				fi
				$BB $BBDISP "status ${MACHINE}.http $COLOR  `date` $LINE"
			else
			#
			# CHECK FOR A SERVER ERROR HIDING HERE... CHECK FOR IT
			#

			#
			# THANKS TO: Douwe Dijkstra <D.Dijkstra@Twinfo.nl>
			# Version 1.04b
			#
				read HTTPVER RC STATUSMESSAGE < $BBTMP/HTTP
				case $RC in
    				'200'|'401'|'302')
         				LINE="Server OK: "
         				COLOR="green"
         				;;
    				*)
         				LINE="Server **NOT** OK: "
         				COLOR="yellow"
         				;;
				esac
				#
				# SMM 1.07c - SEND http HEADERS ALONG
				#
				$BB $BBDISP "status ${MACHINE}.http $COLOR `date` $LINE `cat $BBTMP/HTTP` "
				$RM -f $BBTMP/HTTP
				HTTP=""
			fi
			;;
		\# | BBPAGER | BBDISPLAY | BBNET | dialup | noping  | testip  )
			# These tags do nothing here
			;;
		* )
			# At one point this message will be enabled
			# echo "Invalid bb-hosts directive: $1"
			;;
		esac
		set $line		# RESTORE...
		shift;			# GET NEXT ARGUMENT
	done
done
